;; Basic Settings ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Performance: The emacs default is too low 4k considering that the some of the
;; language server responses are in 800k - 3M range.
(setq read-process-output-max (* 1024 1024)) ;; 1mb

;; Performance: The default setting is too low for lsp-mode's needs due to the
;; fact that client/server communication generates a lot of memory/garbage. You
;; have two options:
(setq gc-cons-threshold 100000000)

;; HACK: Disable warning with "package cl is deprecated".
(setq byte-compile-warnings '(cl-functions))

;; Disable annoying hsplit with emacs help on every program launch.
(setq inhibit-startup-screen t)

;; Place annoying autosave and backup files in fixed dir instead of current dir.
(setq auto-save-file-name-transforms
  `((".*" ,(concat user-emacs-directory "auto-save/") t))) 
(setq backup-directory-alist
  `(("." . ,(expand-file-name
              (concat user-emacs-directory "backups/")))))

;; Package Repositories ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Set up package.el to work with MELPA
(require 'package)
(add-to-list 'package-archives
             '("melpa" . "https://melpa.org/packages/"))
(package-initialize)

;; ENABLE THIS ON FIRST START, OTHERWISE PACKAGE INSTALLATION DOESN'T WORK
(package-refresh-contents)

;; Centered Cursor ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Navigating up or down with j and k keeps the cursor in the middle, and
;; instead moves the text up or down.

;; Variant 1: Builtin Settings
;; Most performant, but going upwards sometimes alternates the middle line with
;; the line one above...
(setq scroll-preserve-screen-position t
      scroll-conservatively 0
      maximum-scroll-margin 0.5
      scroll-margin 99999)

;; Variant 2: centered-cursor-mode (slow)
;; (unless (package-installed-p 'centered-cursor-mode)
;;   (package-install 'centered-cursor-mode))
;; (centered-cursor-mode 1)

;; Variant 3: own alternative (slow)
;; (add-hook 'post-command-hook (lambda () (evil-scroll-line-to-center nil)))

;; Display Keybindings ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Display keybindings on the bottom like in spacemacs.

(unless (package-installed-p 'which-key)
  (package-install 'which-key))
(require 'which-key)
(which-key-mode 1)

;; Set the time delay (in seconds) for the which-key popup to appear. A value of
;; zero might cause issues so a non-zero value is recommended.
(setq which-key-idle-delay 0.2) ; Default 1.0
(setq which-key-idle-secondary-delay 0.02) ; Default nil

;; Set the maximum length (in characters) for key descriptions (commands or
;; prefixes). Descriptions that are longer are truncated and have ".." added.
(setq which-key-max-description-length 40) ; Default 27

;; The maximum number of columns to display in the which-key buffer. nil means
;; don't impose a maximum.
(setq which-key-max-display-columns nil) ; Default nil

;; Language: Agda ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(load-file (let ((coding-system-for-read 'utf-8))
                (shell-command-to-string "agda-mode locate")))

(add-to-list 'auto-mode-alist '("\\.lagda.md\\'" . agda2-mode))
(add-to-list 'auto-mode-alist '("\\.lagda.tex\\'" . agda2-mode))

;; Language: Markdown ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(unless (package-installed-p 'markdown-mode)
  (package-install 'markdown-mode))
(require 'markdown-mode)

;; TODO Settings and Keybindings

;; Fonts ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defface custom-paren-face
  '((t (:foreground "#bbccff")))
  "Face to highlight parentheses."
  :group 'custom-faces)
(defvar custom-paren-face 'custom-paren-face)

(defface custom-infix-face
  '((t (:foreground "#bbccff")))
  "Face to highlight parentheses."
  :group 'custom-faces)
(defvar custom-infix-face 'custom-infix-face)

(defface custom-keyword-small-face
  '((t (:foreground "#60b0ff")))
  "Face to highlight parentheses."
  :group 'custom-faces)
(defvar custom-keyword-small-face 'custom-keyword-small-face)

(defface custom-pattern-face
  '((t (:foreground "#ff8844")))
  "Face to highlight parentheses."
  :group 'custom-faces)
(defvar custom-pattern-face 'custom-pattern-face)

(defface custom-doc-h1-face
  '((t (:inherit font-lock-doc-face :weight ultra-bold :height 2.4)))
  "Face to highlight parentheses."
  :group 'custom-faces)
(defvar custom-doc-h1-face 'custom-doc-h1-face)

(defface custom-doc-h2-face
  '((t (:inherit font-lock-doc-face :weight ultra-bold :height 2.1)))
  "Face to highlight parentheses."
  :group 'custom-faces)
(defvar custom-doc-h2-face 'custom-doc-h2-face)

(defface custom-doc-h3-face
  '((t (:inherit font-lock-doc-face :weight semi-bold :height 1.7)))
  "Face to highlight parentheses."
  :group 'custom-faces)
(defvar custom-doc-h3-face 'custom-doc-h3-face)

(defface custom-doc-h4-face
  '((t (:inherit font-lock-doc-face :weight semi-bold :height 1.4)))
  "Face to highlight parentheses."
  :group 'custom-faces)
(defvar custom-doc-h4-face 'custom-doc-h4-face)

(custom-set-faces
 ;; custom-set-faces was added by Custom.
 ;; If you edit it by hand, you could mess it up, so be careful.
 ;; Your init file should contain only one such instance.
 ;; If there is more than one, they won't work right.
 '(default ((t (:inherit nil :stipple nil :background "#000000" :foreground "#F8F8F2" :inverse-video nil :box nil :strike-through nil :overline nil :underline nil :slant normal :weight normal :height 127 :width normal :foundry "PfEd" :family "DejaVu Sans Mono"))))
 '(agda2-highlight-bound-variable-face ((t (:foreground "white"))))
 '(agda2-highlight-catchall-clause-face ((t nil)))
 '(agda2-highlight-coverage-problem-face ((t (:background "#663333"))))
 '(agda2-highlight-datatype-face ((t (:foreground "#99ff99"))))
 '(agda2-highlight-deadcode-face ((t (:background "#444444"))))
 '(agda2-highlight-function-face ((t (:foreground "#ae81ff"))))
 '(agda2-highlight-inductive-constructor-face ((t (:foreground "#99ff99"))))
 '(agda2-highlight-keyword-face ((t (:foreground "#60b0ff" :weight bold))))
 '(agda2-highlight-number-face ((t (:foreground "#99ff99"))))
 '(agda2-highlight-operator-face ((t (:foreground "#bbccff"))))
 '(agda2-highlight-positivity-problem-face ((t (:background "#663333"))))
 '(agda2-highlight-postulate-face ((t (:inherit nil :foreground "#ae81ff"))))
 '(agda2-highlight-primitive-face ((t nil)))
 '(agda2-highlight-primitive-type-face ((t (:foreground "#99ff99"))))
 '(agda2-highlight-record-face ((t (:foreground "#99ff99"))))
 '(agda2-highlight-symbol-face ((t (:foreground "#60b0ff"))))
 '(agda2-highlight-termination-problem-face ((t (:background "#663333"))))
 '(agda2-highlight-unsolved-constraint-face ((t (:background "#333300"))))
 '(agda2-highlight-unsolved-meta-face ((t (:background "#333300"))))
 '(font-lock-comment-delimiter-face ((t (:foreground "#75715E"))))
 '(font-lock-comment-face ((t (:foreground "#75715E"))))
 '(font-lock-constant-face ((t (:foreground "#ffffff"))))
 '(font-lock-doc-face ((t (:foreground "#75715E"))))
 '(font-lock-function-name-face ((t (:foreground "#bb55ff" :weight black))))
 '(font-lock-keyword-face ((t (:foreground "#60b0ff" :weight ultra-bold))))
 '(font-lock-preprocessor-face ((t (:foreground "#bbccff"))))
 '(font-lock-string-face ((t (:foreground "#e8ff85"))))
 '(font-lock-type-face ((t (:foreground "#ffffff"))))
 '(font-lock-variable-name-face ((t (:foreground "#ffffff"))))
 '(fringe ((t (:background "#000000" :foreground "#F8F8F2"))))
 '(hl-line ((t (:background "#111111"))))
 '(hl-todo ((t (:weight bold))))
 '(line-number ((t (:background "#000000" :foreground "#555555"))))
 '(linum ((t (:background "#000000" :foreground "#555555"))))
 '(markdown-code-face ((t (:foreground "#bbccff"))))
 '(markdown-header-delimiter-face ((t (:foreground "#bbccff"))))
 '(markdown-header-face-1 ((t (:inherit markdown-header-face :foreground "#60b0ff" :weight ultra-bold :height 2.0))))
 '(markdown-header-face-2 ((t (:inherit markdown-header-face :foreground "#60b0ff" :weight ultra-bold :height 1.6))))
 '(markdown-header-face-3 ((t (:inherit markdown-header-face :foreground "#60b0ff" :weight semi-bold :height 1.4))))
 '(markdown-header-face-4 ((t (:inherit markdown-header-face :foreground "#60b0ff" :weight semi-bold :height 1.1))))
 '(markdown-header-face-5 ((t (:inherit markdown-header-face :foreground "#60b0ff" :weight semi-bold))))
 '(markdown-header-face-6 ((t (:inherit markdown-header-face :foreground "#60b0ff" :slant italic))))
 '(markdown-inline-code-face ((t (:inherit markdown-code-face))))
 '(markdown-markup-face ((t (:foreground "#bbccff"))))
 '(markdown-markup-list-face ((t (:foreground "#bbccff"))))
 '(mode-line ((t (:background "#112233" :box (:line-width 1 :color "#000000")))))
 '(mode-line-buffer-id ((t (:foreground "#ffffff" :weight bold))))
 '(mode-line-inactive ((t (:background "#111111" :box (:line-width 1 :color "#000000")))))
 '(powerline-active0 ((t (:background "#071117"))))
 '(powerline-active1 ((t (:background "#071117"))))
 '(powerline-active2 ((t (:background "#071117"))))
 '(powerline-inactive0 ((t (:background "#070707"))))
 '(powerline-inactive1 ((t (:background "#070707"))))
 '(powerline-inactive2 ((t (:background "#070707"))))
 '(show-paren-match ((t (:background "#203040" :foreground "#bbccff" :inverse-video nil :weight ultra-bold))))
 '(vertical-border ((t (:foreground "#111111")))))
(custom-set-variables
 ;; custom-set-variables was added by Custom.
 ;; If you edit it by hand, you could mess it up, so be careful.
 ;; Your init file should contain only one such instance.
 ;; If there is more than one, they won't work right.
 '(package-selected-packages
   '(default-text-scale which-key helm-ls-git helm-descbinds flycheck evil-escape centered-cursor-mode)))
